"""
工具类
"""


import matlab


m_double = matlab.double
m_int16 = matlab.int16
m_int32 = matlab.int32
m_int64 = matlab.int64
m_uint8 = matlab.uint8
m_uint16 = matlab.uint16
m_uint32 = matlab.uint32
m_uint64 = matlab.uint64

