"""
使用示例，本质上是作者做过的各项目与任务中的小规模部分的代码的复制粘贴 \n
示例中的代码会指定编写时的套件版本号，可能存在过时风险，并且对于框架的使用用的是绝对地址包导入 \n
因此，demo中的代码是不存在高度通用性的，但是可以提供包的使用的具体应用场景 \n
同时，如果应用场景碰巧与当前开发场景相同，可以考虑导入其中的部分代码，或直接将示例拷贝为当前开发场景的模板代码，在此基础上进行二次开发
"""
