"""
hagike套件，由hagikehappy制作 \n
`Author`: `hagikehappy` \n
`Institute`: `School of Electronic Science and Engineering, Nanjing University` \n
`Logo`: \n
<img src="__res__/hagikehappy.jpg" style="zoom:30%;" /> \n
"""

